==> web-absence-2.0.tar.gz: absence-v2.0/cgi-bin/AbsenceAuthentication.pm <==
#---------------------------------------------------------------------
# authentication
# $Id: AbsenceAuthentication.pm 91 2009-07-14 15:58:06Z urban $
#---------------------------------------------------------------------

package AbsenceAuthentication;

use FileHandle;
use Fcntl ':flock'; # import LOCK_* constants
use Carp;
use Data::Dumper;

$Data::Dumper::Indent = 1;

my $COOKIE_NAME;

BEGIN {
	# an instance has been specified
	if (exists($ENV{INSTANCE_NAME})) {
        $COOKIE_NAME = "absence-sid-$ENV{INSTANCE_NAME}";
    } else {
        $COOKIE_NAME = 'absence-sid';
    }
}

use AbsenceConfig;
use AbsenceDB;
use AbsenceLog;


use Digest::MD5 qw(md5_base64);

#---------------------------------------------------------------------
# package globals
#---------------------------------------------------------------------

my $AUTH_TYPE		= AbsenceConfig::fetch('auth_type');
my $AUTH			= AbsenceConfig::fetch('authentication');
my $SECRET			= AbsenceConfig::fetch('secret');
my $TOP_PAGE		= AbsenceConfig::fetch('top_page');
my $CRED_SRC		= AbsenceConfig::fetch('credential_src');
my $SESSION_FILE	= AbsenceConfig::fetch('session_file');
my $SESSION_TIMEOUT	= AbsenceConfig::fetch('session_timeout');
my $PW_HASH_FORMAT	= AbsenceConfig::fetch('pw_hash_format');
my $COLOR_MAIN		= AbsenceConfig::fetch('wp_main');
my $COLOR_NOPRIV	= AbsenceConfig::fetch('wp_nopriv');

our $SESSION;

my $DEBUG		= 0;
my $VERSION		= '1.1';
my $AUTH_UID;

#---------------------------------------------------------------------
# class methods
#---------------------------------------------------------------------

sub checkAuthentication
{
	my $cgi = shift;

	$DEBUG && abslog("checkAuthentication()");

	my $hack = AbsenceConfig::fetch('partial_auth_hack');
	(!$AUTH && !$hack) && return undef;

	my $sid = $cgi->cookie($COOKIE_NAME);

	($AUTH && !defined($sid)) && notAuthenticated($cgi);

	$SESSION = AbsenceDB::getSession($sid, 0);
	$DEBUG && abslog("  SESSION=$SESSION");

	(!$AUTH && $hack) && return undef;

	defined($SESSION) || notAuthenticated($cgi);

	if ($SESSION->param('authed')) {
		return $SESSION->param('uid');
	}

	my $cookie = $cgi->cookie(
		-name	=> $COOKIE_NAME,
		-value	=> $SESSION->id(),
	);
	notAuthenticated($cgi, $cookie);
}

sub isAuthed
{
	defined($SESSION) || confess "SESSION undefined";
	return $SESSION->param('authed') ? 1 : 0;
}

sub session
{
	return $SESSION;
}

sub authUid
{
	if (@_) {
		$AUTH_UID = shift;
		my $username = AbsenceDB::getUser(id => $AUTH_UID, 'username');
		defined($username) || die "unable to get username for uid=[$AUTH_UID]";
		AbsenceLog::setAuthUser($username);
	} else {
		return $AUTH_UID;
	}
}

#-----------------------------------------------------------------
# when logout() is called, $SESSION is already defined
#-----------------------------------------------------------------
sub logout
{
	my $q = shift;

	dbg("logout(), SESSION=[$SESSION]");

	my $cook = $q->cookie(
		-name		=> $COOKIE_NAME,
		-value		=> $SESSION->id(),
		-expires	=> '-10y',
	);

	my @header = (-cookie => $cook);

	if ($AUTH_TYPE eq 'http') {
		push(@header,
			-type				=> 'text/html',
			-status				=> '401 Authorization required',
			-nph				=> 1,
			-www_authenticate	=> 'Basic realm="Password Required"',
		);
	}

	print $q->header(@header),
		$q->start_html(
			-title	=> 'Absence: logged off',
			-BGCOLOR	=> $COLOR_MAIN,
			-script		=> "top.location = '$TOP_PAGE';",
		),
		"logged off.",
		$q->end_html();

	$SESSION->delete();
	$SESSION->flush();
}

sub authenticateUser
{
	my $cgi = shift;

	#-----------------------------------------------------------------
	# first find or create a session
	#-----------------------------------------------------------------
	
	dbg("authenticateUser: AUTH=[$AUTH]");

	my $sid = $cgi->cookie($COOKIE_NAME);
	$SESSION = AbsenceDB::getSession($sid, 1);

	my $cookie = $cgi->cookie(
		-name	=> $COOKIE_NAME,
		-value	=> $SESSION->id(),
	);

	if (!$AUTH) {
		if (!AbsenceConfig::fetch('partial_auth_hack')) {
			return (undef, $cookie);
		}
		my $action = $cgi->param('action');
		if (!defined($action)) {
			return (undef, $cookie);
		}
		if ($action eq 'Login') {
			if ($cgi->param('password') eq AbsenceConfig::fetch('partial_auth_hack_pw')) {
				$SESSION->param(authed => 1);
				$SESSION->param(uid => 0);
				$SESSION->flush();
				dbg("add session: uid=[$AUTH_UID], user=[$Q::username], addr=[$ENV{REMOTE_ADDR}]");
				reloadTop($cgi, $cookie);
			} else {
				authenticationFailed($cgi, $cookie);
			}
		}
		elsif ($action eq 'partial_auth') {
			authenticationForm($cgi, 'partial', 'Please enter the SuperUser password', $cookie);
		}
		
		return (undef, $cookie);
	}

	if ($SESSION->param('authed')) {
		return $SESSION->param('uid');
	}

	$SESSION->expire($SESSION_TIMEOUT);

	if ($AUTH_TYPE eq 'http') {
		if (exists($ENV{REMOTE_USER})) {
			$AUTH_UID = AbsenceDB::getUser(username => $ENV{REMOTE_USER}, 'id');
			if (defined($AUTH_UID)) {
				$SESSION->param(authed => 1);
				$SESSION->param(uid => $AUTH_UID);
				$SESSION->flush();
				#return $AUTH_UID;
				reloadTop($cgi, $cookie);
			}
		}
		notAuthenticated($cgi, $cookie);
	} elsif ($AUTH_TYPE eq 'simple') {
		dbg("auth_type = simple");
		if (length($Q::action) && ($Q::action eq 'Login')) {
			$AUTH_UID = checkPassword($Q::username, $Q::password);
			dbg("checkPassword returned [$AUTH_UID]");
			if ($AUTH_UID) {
				$SESSION->param(authed => 1);
				$SESSION->param(uid => $AUTH_UID);
				$SESSION->flush();
				dbg("add session: uid=[$AUTH_UID], user=[$Q::username], addr=[$ENV{REMOTE_ADDR}]");
				reloadTop($cgi, $cookie);
			} else {
				authenticationFailed($cgi, $cookie);
			}
		} else {
			dbg("last clause");
			authenticationForm($cgi, 'normal', undef, $cookie);
		}
	}

	exit;
}

sub reloadTop
{
	my ($q, $cook) = @_;
	print $q->header(
			-cookie	=> $cook,
		),
		$q->start_html(
			-title		=> 'Absence: Authentication OK',
			-BGCOLOR	=> $COLOR_MAIN,
			-script		=> "top.location = '$TOP_PAGE';",
		),
		$q->start_form(
			-action		=> $TOP_PAGE,
			-method		=> 'POST',
			-target		=> '_parent',
		),
		$q->submit(action => 'Authentication OK. Click to Continue...'),
		$q->end_form(),
		$q->end_html();
}

#---------------------------------------------------------------------
# looks up the password for a user and checks the stored password
# against the password passed as parameter.  if the passwords match,
# the user-id is returned.  if no match, returns 0
#---------------------------------------------------------------------
sub checkPassword
{
	my ($user, $pass) = @_;

	if ($CRED_SRC eq 'absence') {
		#-----------------------------------------------------------
		# simple built-in authentication
		#-----------------------------------------------------------
		my $uref = AbsenceDB::getUser(username => $user);
		dbg("checkPassword: uref contents:\n".Dumper($uref)."\n--end--");
		defined($uref) || return 0;
		my $pw = ($PW_HASH_FORMAT eq 'md5')
			? md5_base64($pass)
			: $pass;
		($pw eq $uref->{password}) && return $uref->{id};
	}

	return 0;
}

sub authenticationForm
{
	my ($q, $type, $msg, $cookie) = @_;

	print $q->header(-cookie => $cookie),
		$q->start_html(
			-title		=> 'Absence Authentication',
			-BGCOLOR	=> $COLOR_MAIN,
		);
	if ($msg) { print "<H2>$msg</H2>\n<P>\n"; }
	print "<H2>Please log in</H2>\n",
		$q->start_form(-method => 'GET'),
		'<TABLE>';
	if ($type eq 'normal') {
		print '<TR><TH ALIGN="right">USER:</TH><TD>',
		$q->textfield(
			-name		=> 'username',
			-size		=> 16,
			-maxlength	=> 16,
		),
		qq[</TD></TR>\n];
	}
	print qq[<TR><TH ALIGN="right">PASSWORD:</TH><TD>],
		$q->password_field(
			-name		=> 'password',
			-size		=> 16,
			-maxlength	=> 16,
		),
		"</TD></TR></TABLE><P>",
		$q->submit(action => 'Login'),
		$q->end_form(),
		$q->end_html();
}

sub authenticationFailed
{
	my ($q, $cookie) = @_;

	print $q->header(
				-expires => 'now',
				-cookie	=> $cookie,
			),
		$q->start_html(
			-BGCOLOR	=> $COLOR_NOPRIV,
			-title		=> 'Absence authentication failed',
		),
		$q->h1('Authentication Failed'),
		$q->end_html;
}

sub notAuthenticated
{
	my ($q, $cookie) = @_;

	my @cookie = defined($cookie) ? (-cookie => $cookie) : ();

	print $q->header(
			-expires	=> 'now',
			@cookie,
		),
		$q->start_html(
			-title		=> 'Absence: not authenticated',
			-BGCOLOR	=> $COLOR_MAIN,
		),
		$q->h1('Not Authenticated'),
		$q->end_html;
	exit;
}

sub dbg
{
	my $msg = shift;
	$DEBUG || return;
	abslog("auth: $msg");
}

1;

==> web-absence-2.1.tar.gz: absence-v2.1/cgi-bin/AbsenceAuthentication.pm <==
#---------------------------------------------------------------------
# authentication
# $Id: AbsenceAuthentication.pm 115 2013-12-10 23:45:43Z urban $
# copyright Robert Urban
#---------------------------------------------------------------------

#======================================================================
#    This file is part of Absence.
#
#    Absence is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    Absence is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Absence.  If not, see <http://www.gnu.org/licenses/>.
#======================================================================

package AbsenceAuthentication;

use FileHandle;
use Fcntl ':flock'; # import LOCK_* constants
use Carp;
use Data::Dumper;

$Data::Dumper::Indent = 1;

my $COOKIE_NAME;

BEGIN {
	# an instance has been specified
	if (exists($ENV{INSTANCE_NAME})) {
        $COOKIE_NAME = "absence-sid-$ENV{INSTANCE_NAME}";
    } else {
        $COOKIE_NAME = 'absence-sid';
    }
}

use AbsenceConfig;
use AbsenceDB;
use AbsenceLog;


use Digest::MD5 qw(md5_base64);

#---------------------------------------------------------------------
# package globals
#---------------------------------------------------------------------

my $AUTH_TYPE		= AbsenceConfig::fetch('auth_type');
my $AUTH			= AbsenceConfig::fetch('authentication');
my $SECRET			= AbsenceConfig::fetch('secret');
my $TOP_PAGE		= AbsenceConfig::fetch('top_page');
my $CRED_SRC		= AbsenceConfig::fetch('credential_src');
my $SESSION_FILE	= AbsenceConfig::fetch('session_file');
my $SESSION_TIMEOUT	= AbsenceConfig::fetch('session_timeout');
my $PW_HASH_FORMAT	= AbsenceConfig::fetch('pw_hash_format');
my $COLOR_MAIN		= AbsenceConfig::fetch('wp_main');
my $COLOR_NOPRIV	= AbsenceConfig::fetch('wp_nopriv');
my $JS_RELOAD_CONTROL	= 'function reload_control() { parent.control.location.reload(); }';

our $SESSION;

my $DEBUG		= 0;
my $VERSION		= '2.0.1';
my $AUTH_UID;

#---------------------------------------------------------------------
# class methods
#---------------------------------------------------------------------

sub checkAuthentication
{
	my $cgi = shift;

	$DEBUG && abslog("checkAuthentication()");

	my $hack = AbsenceConfig::fetch('partial_auth_hack');
	(!$AUTH && !$hack) && return undef;

	my $sid = $cgi->cookie($COOKIE_NAME);

	($AUTH && !defined($sid)) && notAuthenticated($cgi);

	$SESSION = AbsenceDB::getSession($sid, 0);
	$DEBUG && abslog("  SESSION=$SESSION");

	(!$AUTH && $hack) && return undef;

	defined($SESSION) || notAuthenticated($cgi);

	if ($SESSION->param('authed')) {
		return $SESSION->param('uid');
	}

	my $cookie = $cgi->cookie(
		-name	=> $COOKIE_NAME,
		-value	=> $SESSION->id(),
	);
	notAuthenticated($cgi, $cookie);
}

sub isAuthed
{
	defined($SESSION) || confess "SESSION undefined";
	return $SESSION->param('authed') ? 1 : 0;
}

sub session
{
	return $SESSION;
}

sub authUid
{
	if (@_) {
		$AUTH_UID = shift;
		my $username = AbsenceDB::getUser(id => $AUTH_UID, 'username');
		defined($username) || die "unable to get username for uid=[$AUTH_UID]";
		AbsenceLog::setAuthUser($username);
	} else {
		return $AUTH_UID;
	}
}

#-----------------------------------------------------------------
# when logout() is called, $SESSION is already defined
#-----------------------------------------------------------------
sub logout
{
	my $q = shift;

	dbg("logout(), SESSION=[$SESSION]");

	my $cook = $q->cookie(
		-name		=> $COOKIE_NAME,
		-value		=> $SESSION->id(),
		-expires	=> '-10y',
	);

	my @header = (-cookie => $cook);

	my @js_params = (
	);

	if ($AUTH_TYPE eq 'http') {
		my $lo_path = AbsenceConfig::fetch('js_dir_rel') . '/logout.js' . '?' . time();
		@js_params = (
			-script		=> { -type => 'JAVASCRIPT', -src => $lo_path },
			-onLoad		=> 'parent.control.document.write(""); clearAuthenticationCache(); parent.location.reload();',
		);
	}
	else {
		@js_params = (
			-script		=> $JS_RELOAD_CONTROL,
			-onLoad		=> 'reload_control();',
		);
	}

	print $q->header(@header),
		$q->start_html(
			-title	=> 'Absence: logged off',
			-BGCOLOR	=> $COLOR_MAIN,
			#-script		=> "top.location = '$TOP_PAGE';",
			@js_params,
		),
		$q->h1({-align => 'center'},"logged off."),
		$q->start_form(
			-action		=> $TOP_PAGE,
			-method		=> 'POST',
			-target		=> '_parent',
		);
	if ($AUTH_TYPE eq 'simple') {
		print '<center>',
			$q->submit(-name => 'go_to_login', -value => 'Login to Absence'),
			'</center>';
	}
	print $q->end_form(), $q->end_html();

	$SESSION->delete();
	$SESSION->flush();
}

sub authenticateUser
{
	my $cgi = shift;

	#-----------------------------------------------------------------
	# first find or create a session
	#-----------------------------------------------------------------
	
	dbg("authenticateUser: AUTH=[$AUTH]");

	my $sid = $cgi->cookie($COOKIE_NAME);
	$SESSION = AbsenceDB::getSession($sid, 1);
	abslog("authenticateUser: sid=[$sid]");

	my $cookie = $cgi->cookie(
		-name	=> $COOKIE_NAME,
		-value	=> $SESSION->id(),
	);

	if (!$AUTH) {
		if (!AbsenceConfig::fetch('partial_auth_hack')) {
			return (undef, $cookie);
		}
		my $action = $cgi->param('action');
		if (!defined($action)) {
			return (undef, $cookie);
		}
		if ($action eq 'Login') {
			if ($cgi->param('password') eq AbsenceConfig::fetch('partial_auth_hack_pw')) {
				$SESSION->param(authed => 1);
				$SESSION->param(uid => 0);
				$SESSION->flush();
				dbg("add session: uid=[$AUTH_UID], user=[$Q::username], addr=[$ENV{REMOTE_ADDR}]");
				reloadTop($cgi, $cookie);
			} else {
				authenticationFailed($cgi, $cookie);
			}
		}
		elsif ($action eq 'partial_auth') {
			authenticationForm($cgi, 'partial', 'Please enter the SuperUser password', $cookie);
		}
		
		return (undef, $cookie);
	}

	if ($SESSION->param('authed')) {
		abslog("session is authed");
		return $SESSION->param('uid');
	}

	$SESSION->expire($SESSION_TIMEOUT);

	if ($AUTH_TYPE eq 'http') {
		if (exists($ENV{REMOTE_USER})) {
			$AUTH_UID = AbsenceDB::getUser(username => $ENV{REMOTE_USER}, 'id');
			if (defined($AUTH_UID)) {
				$SESSION->param(authed => 1);
				$SESSION->param(uid => $AUTH_UID);
				$SESSION->flush();
				#return $AUTH_UID;
				abslog("http authenticated as uid [$AUTH_UID], remote_user=[$ENV{REMOTE_USER}");
				reloadTop($cgi, $cookie);
			}
		}
		notAuthenticated($cgi, $cookie);
	} elsif ($AUTH_TYPE eq 'simple') {
		dbg("auth_type = simple");
		if (length($Q::action) && ($Q::action eq 'Login')) {
			$AUTH_UID = checkPassword($Q::username, $Q::password);
			dbg("checkPassword returned [$AUTH_UID]");
			if ($AUTH_UID) {
				$SESSION->param(authed => 1);
				$SESSION->param(uid => $AUTH_UID);
				$SESSION->flush();
				dbg("add session: uid=[$AUTH_UID], user=[$Q::username], addr=[$ENV{REMOTE_ADDR}]");
				reloadTop($cgi, $cookie);
			} else {
				authenticationFailed($cgi, $cookie);
			}
		} else {
			dbg("last clause");
			authenticationForm($cgi, 'normal', undef, $cookie);
		}
	}

	exit;
}

sub reloadTop
{
	my ($q, $cook) = @_;
	print $q->header(
			-cookie	=> $cook,
		),
		$q->start_html(
			-title		=> 'Absence: Authentication OK',
			-BGCOLOR	=> $COLOR_MAIN,
			-script		=> $JS_RELOAD_CONTROL,
			-onLoad		=> "reload_control(); top.location = '$TOP_PAGE';",
		),
		$q->start_form(
			-action		=> $TOP_PAGE,
			-method		=> 'POST',
			-target		=> '_parent',
		),
		$q->submit(action => 'Authentication OK. Click to Continue...'),
		$q->end_form(),
		$q->end_html();
	exit;
}

#---------------------------------------------------------------------
# looks up the password for a user and checks the stored password
# against the password passed as parameter.  if the passwords match,
# the user-id is returned.  if no match, returns 0
#---------------------------------------------------------------------
sub checkPassword
{
	my ($user, $pass) = @_;

	$DEBUG && abslog("checkPassword: user=$user, pass=[$pass]");

	if ($CRED_SRC eq 'absence') {
		#-----------------------------------------------------------
		# simple built-in authentication
		#-----------------------------------------------------------
		my $uref = AbsenceDB::getUser(username => $user);
		dbg("checkPassword: uref contents:\n".Dumper($uref)."\n--end--");
		defined($uref) || return 0;
		my $pw = ($PW_HASH_FORMAT eq 'md5')
			? md5_base64($pass)
			: $pass;
		($pw eq $uref->{password}) && return $uref->{id};
	}

	return 0;
}

sub authenticationForm
{
	my ($q, $type, $msg, $cookie) = @_;

	print $q->header(-cookie => $cookie),
		$q->start_html(
			-title		=> 'Absence Authentication',
			-BGCOLOR	=> $COLOR_MAIN,
		);
	if ($msg) { print "<H2>$msg</H2>\n<P>\n"; }
	print "<H2>Please log in</H2>\n",
		$q->start_form(-method => 'GET'),
		'<TABLE>';
	if ($type eq 'normal') {
		print '<TR><TH ALIGN="right">USER:</TH><TD>',
		$q->textfield(
			-name		=> 'username',
			-size		=> 16,
			-maxlength	=> 16,
		),
		qq[</TD></TR>\n];
	}
	print qq[<TR><TH ALIGN="right">PASSWORD:</TH><TD>],
		$q->password_field(
			-name		=> 'password',
			-size		=> 16,
			-maxlength	=> 16,
		),
		"</TD></TR></TABLE><P>",
		$q->submit(action => 'Login'),
		$q->end_form(),
		$q->end_html();
}

sub authenticationFailed
{
	my ($q, $cookie) = @_;

	print $q->header(
				-expires => 'now',
				-cookie	=> $cookie,
			),
		$q->start_html(
			-BGCOLOR	=> $COLOR_NOPRIV,
			-title		=> 'Absence authentication failed',
		),
		$q->h1('Authentication Failed'),
		$q->end_html;
}

sub notAuthenticated
{
	my ($q, $cookie) = @_;

	my @cookie = defined($cookie) ? (-cookie => $cookie) : ();

	print $q->header(
			-expires	=> 'now',
			@cookie,
		),
		$q->start_html(
			-title		=> 'Absence: not authenticated',
			-BGCOLOR	=> $COLOR_MAIN,
		),
		$q->h1('Not Authenticated'),
		$q->end_html;
	exit;
}

sub dbg
{
	my $msg = shift;
	$DEBUG || return;
	abslog("auth: $msg");
}

1;
